"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _dateFns = require("date-fns");

var _fs = _interopRequireDefault(require("fs"));

var _messageformatParser = _interopRequireDefault(require("messageformat-parser"));

var _pluralsCldr = _interopRequireDefault(require("plurals-cldr"));

var _pofile = _interopRequireDefault(require("pofile"));

var _querystring = _interopRequireDefault(require("querystring"));

var R = _interopRequireWildcard(require("ramda"));

var _utils = require("../utils");

var getCreateHeaders = function getCreateHeaders() {
  var language = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "no";
  return {
    "POT-Creation-Date": (0, _dateFns.format)(new Date(), "yyyy-MM-dd HH:mmxxxx"),
    "Mime-Version": "1.0",
    "Content-Type": "text/plain; charset=utf-8",
    "Content-Transfer-Encoding": "8bit",
    "X-Generator": "@lingui/cli",
    Language: language
  };
}; // Attempts to turn a single tokenized ICU plural case back into a string.


var stringifyICUCase = function stringifyICUCase(icuCase) {
  return icuCase.tokens.map(function (token) {
    if (typeof token === "string") {
      return token;
    } else if (token.type === "octothorpe") {
      return "#";
    } else if (token.type === "argument") {
      return "{" + token.arg + "}";
    } else {
      console.warn("Unexpected token \"".concat(token, "\" while stringifying plural case \"").concat(icuCase, "\". Token will be ignored."));
      return "";
    }
  }).join("");
};

var serialize = R.compose(R.values, R.mapObjIndexed(function (message, key) {
  var item = new _pofile.default.Item();
  item.msgid = key; // Depending on whether custom ids are used by the developer, the (potential plural) "original", untranslated ICU
  // message can be found in `message.message` or in the item's `key` itself.

  var icuMessage = message.message || key; // Quick check to see if original message is a plural localization.

  if (/^{.*, plural, .*}$/.test(icuMessage)) {
    try {
      var _ICUParser$parse = _messageformatParser.default.parse(icuMessage),
          _ICUParser$parse2 = (0, _slicedToArray2.default)(_ICUParser$parse, 1),
          messageAst = _ICUParser$parse2[0]; // TODO: Check for multiple plural cases and output error.
      // Store placeholder that is pluralized upon to allow restoring ICU format later.


      item.msgctxt = _querystring.default.encode({
        pluralize_on: messageAst.arg
      }); // For the plural key, simple append "_plural" to the original message key (might be the full message or its id)

      item.msgid_plural = key + "_plural"; // If there is a translated value, parse that instead of the original message to prevent overriding localized
      // content with the original message. If there is no translated value, don't touch msgstr, since marking item as
      // plural (above) already causes `pofile` to automatically generate `msgstr[0]` and `msgstr[1]`.

      if (message.translation) {
        try {
          var _ICUParser$parse3 = _messageformatParser.default.parse(message.translation),
              _ICUParser$parse4 = (0, _slicedToArray2.default)(_ICUParser$parse3, 1),
              ast = _ICUParser$parse4[0];

          item.msgstr = ast.cases.map(stringifyICUCase);
        } catch (e) {
          console.error("Error parsing translation ICU", e);
        }
      }
    } catch (e) {
      console.error("Error parsing message ICU:", e);
    }
  } else {
    item.msgstr = [message.translation];
  }

  item.comments = message.comments || [];
  item.extractedComments = message.comment ? [message.comment] : [];
  item.references = message.origin ? message.origin.map(_utils.joinOrigin) : []; // @ts-ignore: Figure out how to set this flag

  item.obsolete = message.obsolete;
  item.flags = message.flags ? R.fromPairs(message.flags.map(function (flag) {
    return [flag, true];
  })) : {};
  return item;
}));
var getMessageKey = R.prop("msgid");
var getTranslations = R.prop("msgstr");
var getExtractedComments = R.prop("extractedComments");
var getTranslatorComments = R.prop("comments");
var getOrigins = R.prop("references");
var getFlags = R.compose(R.map(R.trim), R.keys, R.dissoc("obsolete"), // backward-compatibility, remove in 3.x
R.prop("flags"));
var isObsolete = R.either(R.path(["flags", "obsolete"]), R.prop("obsolete"));
var deserialize = R.map(R.applySpec({
  translation: R.compose(R.head, R.defaultTo([]), getTranslations),
  comment: R.compose(R.head, R.defaultTo([]), getExtractedComments),
  comments: function comments(item) {
    return R.concat(getTranslatorComments(item), R.tail(getExtractedComments(item)));
  },
  obsolete: isObsolete,
  origin: R.compose(R.map(_utils.splitOrigin), R.defaultTo([]), getOrigins),
  flags: getFlags
}));

var validateItems = function validateItems(lang) {
  // .po plurals are numbered 0-N and need to be mapped to ICU plural classes ("one", "few", "many"...). Different
  // languages can have different plural classes (some start with "zero", some with "one"), so read that data from CLDR.
  var pluralForms = [];

  if (lang) {
    pluralForms = _pluralsCldr.default.forms(lang); // Plurals may be `null` if lang is not found. As long as no plural is used, don't report an error.
  }

  return R.forEach(function (item) {
    var translations = getTranslations(item);
    var translationCount = translations.length;
    var messageKey = getMessageKey(item);

    if (translationCount > 1) {
      // msgid_plural must be set, but its actual value is not important.
      if (!item.msgid_plural) {
        console.warn("Multiple translations for item with key \"%s\" but missing 'msgid_plural' is not supported and it will be ignored.", messageKey);
        return;
      } // If all translations are empty, ignore item.


      if (item.msgstr.every(function (str) {
        return str.length === 0;
      })) {
        return;
      }

      if (pluralForms === null) {
        console.warn("Multiple translations for item with key \"%s\"\xA0in language \"".concat(lang, "\", but no plural cases were found. ") + "This prohibits the translation of .po plurals into ICU plurals. Pluralization will not work for this key.", messageKey);
        return;
      }

      var pluralCount = pluralForms.length;

      if (translationCount > pluralCount) {
        console.warn("More translations provided for item with key \"%s\" (".concat(translationCount, ") in language \"").concat(lang, "\" than there are plural cases available (").concat(pluralCount, "). ") + "This will result in not all translations getting picked up.", messageKey);
      } // Map each msgstr to a "<pluralform> {<translated_string>}" entry, joined by one space.


      var pluralClauses = item.msgstr.map(function (str, index) {
        return pluralForms[index] + " {" + str + "}";
      }).join(" "); // Find out placeholder name from item's message context, defaulting to "count".

      var _querystring$parse = _querystring.default.parse(item.msgctxt),
          pluralizeOn = _querystring$parse.pluralize_on;

      if (!pluralizeOn) {
        console.warn("Unable to determine plural placeholder name for item with key \"%s\" (should be stored in \"msgctxt\"), assuming \"count\".", messageKey);
        pluralizeOn = "count";
      }

      item.msgstr = ["{" + pluralizeOn + ", plural, " + pluralClauses + "}"];
    }
  });
};

var indexItems = R.indexBy(getMessageKey);
var _default = {
  catalogExtension: ".po",
  write: function write(filename, catalog, options) {
    var po;

    if (_fs.default.existsSync(filename)) {
      var raw = _fs.default.readFileSync(filename).toString();

      po = _pofile.default.parse(raw);
    } else {
      po = new _pofile.default();
      po.headers = getCreateHeaders(options.locale);
      po.headerOrder = R.keys(po.headers);
    }

    po.items = serialize(catalog);
    (0, _utils.writeFileIfChanged)(filename, po.toString());
  },
  read: function read(filename) {
    var raw = _fs.default.readFileSync(filename).toString();

    return this.parse(raw);
  },
  parse: function parse(raw) {
    var po = _pofile.default.parse(raw);

    validateItems(po.headers.Language)(po.items);
    return deserialize(indexItems(po.items));
  }
};
exports.default = _default;